# user-defined exception
class PasswordTooShort(Exception):
    def __init__(self):
        pass

class PasswordTooLong(Exception):
    def __init__(self):
        pass

class User():
    """
    Includes User definition and functions.
    
    Parameters
    ----------
    userCount: count the number of User created
    users: list storing all current users
    usernames: list storing all user names
    """
    
    userCount = 1 
    
    users = [] 

    usernames = []
    
    def __init__(self, user, password):
        """
        Define the class user
        
        Parameters
        ----------
        user: the name of user
        id: the id of user (unique)
        password: personal password for login (private variable)
        """
        if user not in self.usernames:
            self.user = user
            self.id = User.userCount
            self._password = password # password is private 
            #print('Username: {}'.format(self.user))
            User.userCount += 1
            self.users.append(self)
            self.usernames.append(user)
        self.user = user
        
    def displayUser(self):
        """
        Display user info
        
        Returns
        ----------
        The string showing username.
        """
        return f'Username: {self.user}'
        
        # change password
    @property
    def password(self):
        return self._password
    @password.setter
    def password(self, value):
        # error and exception handler - 5
        try:
            if len(value) < 6:
                raise PasswordTooShort
            elif len(value) > 12:
                raise PasswordTooLong
        except PasswordTooShort:
            print('The password is too short')
        except PasswordTooLong:
            print('The password is too long')
        except:
            print('Other error occurred')
        else:
            self._password = value

# users
user1 = User('madison13g', 'password1')
user2 = User('amethyst1016', 'password2')
user3 = User('khaladhasan', 'khalad1')
user4 = User('jeffandrews', 'jeff1')
user5 = User('christeldeas', 'christel1')
user6 = User('gemarodriguezperez', 'gema1')
user7 = User('ifeomaadaji', 'ifeoma1')
user8 = User('johnthompson', 'john1')
user9 = User('emeliegustafsson', 'emelie1')
user10 = User('patricialassere', 'patricia1')


class Topic:
    """
    Includes topic class definition and functions.
    
    Parameters
    ----------
    index: count the number of User created
    topics: list storing all current topics
    topics_name: list storing all topic names
    """
    
    topics = [] # store all topic objects
    topics_name = [] # store all topic names (tname)
    index = 1
    def __init__(self, tname, user):
        """
        Define class topic
        
        Parameters
        ----------
        tname: the name of topic
        tid: the id of topic (unique)
        tag: tag/label of topic (private variable)
        """
        if tname not in Topic.topics_name:
            self.tid = Topic.index
            Topic.index += 1
            self.tname = tname
            Topic.topics.append(self)
            Topic.topics_name.append(tname)
            self.user = user
            self._tag=""
    @property
    def tag(self):
        return self._tag
    @tag.setter
    def tag(self, tag):
        # error and exception handler - 4
        try:
            tag = str(tag)
        except TypeError:
            print('TypeError occurred')
        except:
            print('Other error occurred')
        else:
            self._tag = tag
    
    def show(self):
        """
        Display topic info
        
        Returns
        ----------
        The string showing topic id and name.
        """
        return f'Topic id: {self.tid} Topic name: {self.tname}'
    
    def search(tag):
        """
        Display all topic with a given tag
        
        Returns
        ----------
        The string showing all topics' id and name under a given tag or indicating there is no topic under the given tag.
        """
        exist = 0
        for i in Topic.topics:
            if i.tag == tag:
                exist = 1
                return i.show()
        if exist == 0:
            return('[No topic under this tag.]')

# topics
topic1 = Topic('DATA533 Collaborative Software Development', 'khaladhasan')
topic1.tag = 'DATA533'
topic2 = Topic('DATA571 Resampling and Regularization', 'jeffandrews')
topic2.tag = 'DATA571'
topic3 = Topic('DATA532 Data Structure and Algorithm', 'gemarodriguezperez')
topic3.tag = 'DATA532'
topic4 = Topic('DATA540 Databases and Data Retrievel', 'ifeomaadaji')
topic4.tag = 'DATA540'
topic5 = Topic('DATA570 Predictive Modelling', 'johnthompson')
topic5.tag = 'DATA570'
topic6 = Topic('DATA543 Data Collection', 'emeliegustafsson')
topic6.tag = 'DATA543'
topic7 = Topic('DATA553 Privacy, Security and Professional Ethics', 'patricialassere')
topic7.tag = 'DATA553'
topic8 = Topic('DATA533 Project', 'amethyst1016')
topic8.tag = 'DATA533'
topic9 = Topic('DATA533 Project Group 8', 'madison13g')
topic9.tag = 'DATA533'
topic10 = Topic('General', 'christeldeas')
topic10.tag = 'MDS'


from Content.topic import Topic

class Post(Topic):
    """
    Includes post definition and functions.
    
    Parameters
    ----------
    index: count the number of post created
    posts: list storing all current posts
    """
    
    posts = []
    index = 1
    def __init__(self, topicid, detail, user):
        """
        Define the class post (inheritance Topic)
        
        Parameters
        ----------
        tid: id of topic which this post under
        detail: content of the post
        user: name of user who write this post
        comments: list of comments under this post
        like: number of likes this post has
        pidL id of the post (unique)
        """
        topic = Topic.topics_name[topicid-1]
        self.tid = topicid
        Topic.__init__(self, topic, user='')
        Post.posts.append(self)
        self.detail = detail
        self.user = user
        self.comments = []
        self.like = 0
        self.pid = Post.index
        Post.index += 1
        
    def add_comment(self, comment):
        ## Function to add a comment to a post
        
        # error and exception handler - 2
        try:
            comment = str(comment)
        except TypeError: 
            print('TypeError occurred')
        except:
            print('Other error occurred')
        else:
            self.comments.append(comment)
        
    def add_like(self, num=1):
        # Function to add a like to a post
        # error and exception handler - 3
        try:
            num = int(num)
        except ValueError:
            print('ValueError occurred')
        except:
            print('Other error occurred')
        else:
            self.like += num
        
    def check(self):
        """
        Display post info of likes and comments, by quantity.
        
        Returns
        ----------
        The string showing how many likes and comments this post have.
        """
        return f'This post has {self.like} like and {len(self.comments)} comments.'
    
    def show_post(self):
        """
        Display post info
        
        Returns
        ----------
        The string showing post id, detail, writer and comments or indicating there is no comment under this post.
        """
        print(f'Post id: {self.pid}\n{self.detail} ----- by {self.user}')
        if self.comments:
            print('Comments: ')
            for i in self.comments:
                print(i)
        else:
            print('[Currently no comment.]')
            
    def search_all_post(topicid):
        """
        Display posts under a given topic
        
        Returns
        ----------
        The string showing posts under a given topic or indicating there is no post under the given topic.
        """
        exist = 0
        for i in Post.posts:
            if i.tid == topicid:
                exist = 1
                i.show_post()
                print('·'*20)
        if exist == 0:
            print('[No post under this topic.]')
            
    def search_post(keyword):
        """
        Display post containing a given keyword
        
        Returns
        ----------
        The string showing posts with a given keyword or indicating there is no post containing the given keyword.
        """
        exist = 0
        for i in Post.posts:
            if keyword in i.detail:
                exist = 1
                i.show_post()
                i.check()
                print('·'*20)
        if exist == 0:
            print('[No post with this keyword.]')

# posts
user1 = 'amethyst1016'
post1 = Post(10, 'Winter break starts on December 23rd!', user1)
post1.add_like(num=10)
post1.add_comment('Wow!')
post1.add_comment('Two more weeks to go!')
post2 = Post(8, 'Project step 1 due on December 14th.', user1)
post2.add_like(num=2)
post2.add_comment('Already finished.')
post3 = Post(6, 'Final quiz on December 19th.', 'emeliegustafsson')
post3.add_like(num=4)
post3.add_comment('Test on lecture5 to lecture7 inclusive.')
post4 = Post(2, 'Final quiz on December 21st.','jeffandrews')
post4.add_like(num=3)
post4.add_comment('Includes MCQ and short answers.')
post5 = Post(1, 'Final quiz on December 20th', 'khaladhasan')
post5.add_like(num=6)


from .user import User

class Admin(User):
    """
    Includes Administrator definition and functions.
    
    Parameters
    ----------
    admins: list storing all administrators
    """
    
    admins = []
    
    def __init__(self, user, password, admincode):
        """
        Define the class administrator (inheritance User)
        
        Parameters
        ----------
        amincode: like administrator id (unique)
        """
        User.__init__(self, user, password)
        self.admincode = admincode
        Admin.admins.append(self)
        
    def displayUser(self):
        """
        Display administrator info
        
        Returns
        ----------
        The string showing username and admincode.
        """
        return f'{User.displayUser(self)} Admin code: {self.admincode}'
        
    # check number of users
    def displayCount(self):
        """
        Display the number of current users
        
        Returns
        ----------
        The string showing the number of current users.
        """
        return f'There are {len(User.users)} users'
    
    # print list of current users
    def userlist(self):
        """
        Display all user info
        
        Returns
        ----------
        The string showing all users' id and username.
        """
        # error and exception handler - 6
        try:
            for i in User.users:
                print("Userid: {} Username: {}".format(i.id, i.user))
        except IndexError:
            print('IndexError occurred')
        except:
            print('Other error occurred')
            
# administrators
admin1 = Admin('Dan', 'password3', 'A1')
admin2 = Admin('Amethyst', 'password4', 'A2')
admin3 = Admin('Madison', 'password5', 'A3')
admin4 = Admin('Abigail', 'password6', 'A4')
admin5 = Admin('Linda', 'password7', 'A5')
admin6 = Admin('Tom', 'password8', 'A6')